#include "blacktabplugin.h"

#include <KPluginFactory>
#include <KPluginLoader>
#include <KLocale>
#include <KAction>
#include <KActionCollection>
#include <KAboutData>
#include <KDebug>
#include <KStandardDirs>
#include <KStandardAction>
#include <KMessageBox>
#include <KActionMenu>
#include <KConfigDialog>
#include <QGraphicsWidget>

#include "settings.h"

static QString readStartUrlFromProfile();

static const KAboutData aboutdata("blacktabplugin", 0, ki18n("BlackTab Settings") , "0.2.1" );

K_PLUGIN_FACTORY(blacktabPluginFactory, registerPlugin<blacktabPlugin>(); )
K_EXPORT_PLUGIN(blacktabPluginFactory(aboutdata))


blacktabPlugin::blacktabPlugin(QObject *parent, const QVariantList &args)
        : KParts::Plugin(parent)
{
    Q_UNUSED(args);

    setComponentData(blacktabPluginFactory::componentData());

    m_Part = dynamic_cast<KParts::ReadOnlyPart *>(parent);
    if (! m_Part)
    {
        kDebug() << "Unable to get KHTMLPart" << endl;
        return;
    }

    // Connect to web page open
    connect(m_Part, SIGNAL(started(KIO::Job*)), this, SLOT(started(KIO::Job*)));

    kDebug() << "blacktab created" << endl;
    kDebug() << "Home Page = :" << readStartUrlFromProfile() << ":" << endl;


    // Config Action
    QAction *actConfig = actionCollection()->addAction("configure_blacktab");
    actConfig->setText(i18n("&Configure BlackTab"));
    
    // Setup our menu
    KActionMenu*menu = new KActionMenu("BlackTab", actionCollection());
    actionCollection()->addAction("tools_blacktab", menu);
    connect(actConfig, SIGNAL(triggered(bool)), this, SLOT(showConfig()));
    menu->addAction(actConfig);
    
    NewTabHandlerSingleton::Init(this);
}

blacktabPlugin::~blacktabPlugin()
{
}


static QString readStartUrlFromProfile()
{
    const QString blank = "about:blank";
    const QString profile = KStandardDirs::locate("data", QLatin1String("konqueror/profiles/webbrowsing"));
    if (profile.isEmpty())
        return blank;
    KConfig cfg(profile, KConfig::SimpleConfig);
    KConfigGroup profileGroup(&cfg, "Profile");
    const QString rootItem = profileGroup.readEntry("RootItem");
    if (rootItem.isEmpty())
        return blank;
    if (rootItem.startsWith("View")) {
        const QString prefix = rootItem + '_';
        const QString urlKey = QString("URL").prepend(prefix);
        return profileGroup.readPathEntry(urlKey, blank);
    }
    // simplify the other cases: whether root is a splitter or directly the tabwidget,
    // we want to look at the first view inside the tabs, i.e. ViewT0.
    return profileGroup.readPathEntry("ViewT0_URL", blank);
}

static const char* DEFAULT_HOMEPAGE = "http://www.kde.org";

static QString getHomeUrlFromProfile()
{
    KSharedConfig::Ptr m_pConfig = KSharedConfig::openConfig("konquerorrc", KConfig::NoGlobals);
    KConfigGroup userSettings(m_pConfig, "UserSettings");
    QString url = userSettings.readEntry("HomeURL", DEFAULT_HOMEPAGE);

    return url;
}

void OpenNewTabPage(KParts::ReadOnlyPart *part)
{
    int newTab = Settings::newtab();
    kDebug() << "NewTab action = " << newTab << endl;

    KUrl url;
    switch (newTab)
    {
    case 0:
        kDebug() << "Show Konqueror Start Page" << endl;
        url = readStartUrlFromProfile();
        break;

    case 1:
        kDebug() << "Show Introduction Page" << endl;
        url = "about:konqueror";
        break;

    case 2:
        kDebug() << "Show Home Page" << endl;
        url = getHomeUrlFromProfile();
        break;

    case 3:
        kDebug() << "Show Blank Page" << endl;
        url = "about:blank";
        break;

    case 4:
        kDebug() << "Show Bookmarks" << endl;
        url = "bookmarks:/";
        break;

    default:
        kDebug() << "Unknown - do nothing" << endl;
        break;
    }
    kDebug() << "Url = " << url << endl;
    emit part->browserExtension()->openUrlRequest(url);
}

void blacktabPlugin::started(KIO::Job *job)
{
    Q_UNUSED(job);

    // One shot
    disconnect(m_Part, SIGNAL(started(KIO::Job*)), this, SLOT(started(KIO::Job*)));
    kDebug() << "Job started, URL = " << m_Part->url() << endl;

    NewTabHandlerSingleton::s_LastPart = m_Part;
}


void blacktabPlugin::showConfig()
{
    kDebug() << "Configure blacktab" << endl;
    // The preference dialog is derived from config.ui
    //
    // compare the names of the widgets in the .ui file
    // to the names of the variables in the .kcfg file
    //avoid to have 2 dialogs shown
    if ( KConfigDialog::showDialog ( "settings" ) )
    {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog ( NULL, "settings", Settings::self() );
    QWidget *generalSettingsDlg = new QWidget;
    ui_configdialog.setupUi ( generalSettingsDlg );
    dialog->addPage ( generalSettingsDlg, i18n ( "General" ), "package_setting" );
    //connect ( dialog, SIGNAL ( settingsChanged ( QString ) ), m_view, SLOT ( settingsChanged() ) );
    dialog->setAttribute ( Qt::WA_DeleteOnClose );
    dialog->show();

}

///////////////////////////
// NewTabHandlerSingleton
NewTabHandlerSingleton	*NewTabHandlerSingleton::self = NULL;
KParts::ReadOnlyPart 	*NewTabHandlerSingleton::s_LastPart = NULL;
QAction			*NewTabHandlerSingleton::lastActNewTab = NULL;

NewTabHandlerSingleton::NewTabHandlerSingleton()
{
  self =  this;
}

void NewTabHandlerSingleton::Init(blacktabPlugin *plugin)
{
  Q_UNUSED(plugin);
  
  if (self == NULL)
    new NewTabHandlerSingleton();
  
  // Attempt to connect to newtab event
  kDebug() << "Looking for newtab action" << endl;
  QList<KActionCollection *>::const_iterator it =KActionCollection::allCollections().begin();
  while (it != KActionCollection::allCollections().constEnd())
  {
      KActionCollection *ac = *it;
      for (int i = 0; i < ac->count(); i++)
      {
	QAction *act = ac->action(i);
	if (act->objectName() == "newtab")
	{
	  kDebug() << "FOUND NEWTAB ACTION, ObjectName = " << act->objectName() << endl;
	  connect(act , SIGNAL(triggered(bool)), self ,SLOT(slotNewTab(bool)), Qt::UniqueConnection);
	}
      }
      it++;
  }
}

void NewTabHandlerSingleton::slotNewTab(bool checked)
{
    Q_UNUSED(checked);

    kDebug() << "NEWTAB TRIGGER" << endl;
    KParts::ReadOnlyPart *p = s_LastPart;
    s_LastPart = NULL;
    if (p)
        OpenNewTabPage(p);
}


